#!/usr/bin/perl
##Combined Toolset for Pombe
#JAE--sample SVN 
#Depdencencies
use strict;
use Bio::SeqIO;
use Bio::PrimarySeq;
use POSIX;

#Environmental Variables
my ($wdir, $readfile1, $readfile2, $BTpath, $SAMpath, $refGen, $chroms, $WTname, $MUTname, $cutoff, $minimum, $buffer, $step, $windowsize, $ratcutoff);

#Read config
open(SETTINGS,"config.txt") || die("Could not open config\n");
while(<SETTINGS>)
{
	chomp;
	if ($_ =~ m/(.+?)=(.+)/) {
		my ($var, $set) = ($1, $2);
		if ($var eq "[Working Directory]") {$wdir = $set;}
		if ($var eq "[WT FASTQ Read File]") {$readfile1 = $set;}
		if ($var eq "[MUT FASTQ Read File]") {$readfile2 = $set;}
		if ($var eq "[Bowtie Path]") {$BTpath = $set;}
		if ($var eq "[Samtools Path]") {$SAMpath = $set;}
		if ($var eq "[Reference Genome Name]") {$refGen = $set;}
		if ($var eq "[Number Chromosomes]") {$chroms = $set;}
		if ($var eq "[Default Name WT]") {$WTname = $set;}
		if ($var eq "[Default Name MUT]") {$MUTname = $set;}
		if ($var eq "[SNP Min Depth]") {$minimum = $set;}
		if ($var eq "[SNP Min Identity]") {$cutoff = $set;}
		if ($var eq "[Zero Flank Step]") {$buffer = $set;}
		if ($var eq "[Zero Flank Seed Length]") {$step = $set;}
		if ($var eq "[Copy Num Averaging Window]") {$windowsize = $set;}
		if ($var eq "[Copy Num Ratio Cutoff]") {$ratcutoff = $set;}
	}
}
close(SETTINGS);

#Main Program
my $cmdswitch = $ARGV[0];
if (($cmdswitch =~ /-\?/ ) || ($cmdswitch eq "") || ($cmdswitch =~ /-help/ )) {
	print "Usage:\n\t-build : Prepare genome from embl files\n\t-all : perform all following routines\n\t-align : Prepare alignments of WT and MUT reads to built genome\n\t-stat : Report basic statistics of alignments\n\t-SNP : call SNPs and prepare zero coverage report\n\t-flanks : Parse zero coverage and identify flanking sequence seeds\n\t-gaps : Identify gap spanning reads from sequence flanks\n\t-AnCopy : Estimate copy numbers of all geneome features for WT and MUT\n\t-InCopy : Scan genome (context insensitive) for changes in region copy number\n\n";
	exit;
}
if ($cmdswitch =~ /-build/ ) {
	refbuild();
	system("${BTpath}bowtie-build ${BTpath}genomes/${refGen}.fasta $refGen");
	system("mv -f *.ebwt ${BTpath}indexes");
	exit;
}
elsif ($cmdswitch =~ /-align/ ) {
	RAlign($readfile1,$WTname,"Parent");
	RAlign($readfile2,$MUTname,"Mutant");
	exit;
}
elsif ($cmdswitch =~ /-SNP/ ) {
	my $refin = $wdir . $WTname . ".pileup";
	my $mutin = $wdir . $MUTname . ".pileup";
	SNPcall($refin, $mutin, $cutoff, $minimum);
	my $anoin = $wdir . $MUTname . ".SNPS";
	Annotate($anoin);
	exit;
}
elsif ($cmdswitch =~ /-flanks/) {
	my $coordinates = $wdir . $MUTname . ".zero";
	my $seqfile = $BTpath . "genomes/" . $refGen . ".fasta";
	flankclip($coordinates, $seqfile, $buffer, $step, $chroms);
	exit;
}
elsif ($cmdswitch =~ /-gaps/) {
	my $seedfile = $wdir . $MUTname . ".flanks";
	my $fastq = $BTpath . "/reads/" . $readfile2;
	searchgapreads($seedfile, $fastq);
	exit;
}
elsif ($cmdswitch =~ /-stat/) {
	my $pileupfile = $wdir . $WTname . ".pileup";
	AlignmentStat($pileupfile, 1);
	$pileupfile = $wdir . $MUTname . ".pileup";
	AlignmentStat($pileupfile, 1);
	exit;
}
elsif ($cmdswitch =~ /-AnCopy/) {
	my $pileupfile1 = $wdir . $WTname . ".pileup";
	my $pileupfile2 = $wdir . $MUTname . ".pileup";
	print "\nFetching AVG Read Depth $WTname...\n";
	my $AVG1 = AlignmentStat($pileupfile1, 0);
	print "\nFetching AVG Read Depth $MUTname...\n";
	my $AVG2 = AlignmentStat($pileupfile2, 0);
	AnnotCopy($AVG1, $AVG2, $pileupfile1, $pileupfile2);
	exit;
}
elsif ($cmdswitch =~ /-InCopy/) {
	my $pileupfile1 = $wdir. $WTname . ".pileup";
	my $pileupfile2 = $wdir. $MUTname . ".pileup";
	print "\nFetching AVG Read Depth $WTname...\n";
	my $AVG1 = AlignmentStat($pileupfile1, 0);
	print "\nFetching AVG Read Depth $MUTname...\n";
	my $AVG2 = AlignmentStat($pileupfile2, 0);
	IndCopy($AVG1, $AVG2, $pileupfile1, $pileupfile2, $windowsize, $ratcutoff);
	exit;
}
elsif ($cmdswitch =~ /-all/) {
	RAlign($readfile1,$WTname,"Parent");
	RAlign($readfile2,$MUTname,"Mutant");
	my $refin = $wdir . $WTname . ".pileup";
	my $mutin = $wdir . $MUTname . ".pileup";
	SNPcall($refin, $mutin, $cutoff, $minimum);
	my $anoin = $wdir . $MUTname . ".SNPS";
	Annotate($anoin);
	my $coordinates = $wdir . $MUTname . ".zero";
	my $seqfile = $BTpath . "genomes/" . $refGen . ".fasta";
	flankclip($coordinates, $seqfile, $buffer, $step, $chroms);
	my $seedfile = $wdir . $MUTname . ".flanks";
	my $fastq = $BTpath . "/reads/" . $readfile2;
	searchgapreads($seedfile, $fastq);
	my $pileupfile1 = $wdir . $WTname . ".pileup";
	my $pileupfile2 = $wdir . $MUTname . ".pileup";
	print "\nFetching AVG Read Depth $WTname...\n";
	my $AVG1 = AlignmentStat($pileupfile1, 0);
	print "\nFetching AVG Read Depth $MUTname...\n";
	my $AVG2 = AlignmentStat($pileupfile2, 0);
	AnnotCopy($AVG1, $AVG2, $pileupfile1, $pileupfile2);
	IndCopy($AVG1, $AVG2, $pileupfile1, $pileupfile2, $windowsize, $ratcutoff);
	exit;
}

##Start of Subroutines

##Basic ReverseCompliment of sequence string
sub reversecomp {
	my $seq = shift;
	my $revcom = reverse($seq);
	$revcom =~ tr/ACGTacgt/TGCAtgca/;
	return $revcom;
}

##For calling sequence differences from both reference and WT/MUT
sub SNPcall {
	
my $refin = shift; #WT file
my $mutin = shift; #mutant file
my $cutoff = shift; #fraction identity of reads required for called base
my $minimum = shift; #minimum read depth required for called base
	
my %cr;
my %cm;
my @cmm;
my @cmr;

my $snpfile = $wdir . $MUTname . ".SNPS";
open(SNPFILE,">$snpfile");
print "\nFraction: $cutoff | Minimum: $minimum\nReading $refin\n";
print SNPFILE "$refin | Fraction: $cutoff | Minimum: $minimum\n";

open(RFILEIN,$refin) || die("Could not open ", $refin);
while(<RFILEIN>)
{
	my @elements = split(/\t/, $_);
	my $conc = "N";	
	my $chrom = shift @elements;
	my $pos = shift @elements;
	my $ref = shift @elements;
	my $v0 = shift @elements;
	my $v1 = shift @elements;
	my $v2 = shift @elements;
	my $v3 = shift @elements;
	my $readnum = shift @elements;
	my $matchstr = shift @elements;
	my $count = 0;
	#checks concensus count against read count.  Defines base if over Cutoff Fraction reads agree.
	
	my $countR = 0;
	$countR++ while $matchstr =~ m/[\.\,]/g;
	if ($countR >= $cutoff * $readnum) {$conc = $ref; $count = $countR;}
			
	my $countA = 0;
	$countA++ while $matchstr =~ m/A/gi;
	if ($countA >= $cutoff * $readnum) {$conc = "A"; $count = $countA;}
		
	my $countG = 0;
	$countG++ while $matchstr =~ m/G/gi;
	if ($countG >= $cutoff * $readnum) {$conc = "G"; $count = $countG;}
		
	my $countC = 0;
	$countC++ while $matchstr =~ m/C/gi;
	if ($countC >= $cutoff * $readnum) {$conc = "C"; $count = $countC;}
	
	my $countT = 0;
	$countT++ while $matchstr =~ m/T/gi;
	if ($countT >= $cutoff * $readnum) {$conc = "T"; $count = $countT;}
	
	if ($chrom =~ m/(\d+)/) { $chrom = $1; }
	
	#Store entry in appropriate chromosome.
	if ($chrom =~ m/mito/) {@cmr[$pos] = "$ref\t$conc\t$readnum\t$count\t$v1"; next;}
	$cr{$chrom}[$pos] = "$ref\t$conc\t$readnum\t$count\t$v1";
	
}
close(RFILEIN);
print "Reading $mutin\n";
open(MFILEIN,$mutin) || die("Could not open ", $mutin);
while(<MFILEIN>)
{
	my @elements = split(/\t/, $_);
	my $conc = "N";	
	my $chrom = shift @elements;
	my $pos = shift @elements;
	my $ref = shift @elements;
	my $v0 = shift @elements;
	my $v1 = shift @elements;
	my $v2 = shift @elements;
	my $v3 = shift @elements;
	my $readnum = shift @elements;
	my $matchstr = shift @elements;
	my $count = 0;
	
	my $countR = 0;
	$countR++ while $matchstr =~ m/[\.\,]/g;
	if ($countR >= $cutoff * $readnum) {$conc = $ref; $count = $countR;}
	
	my $countA = 0;
	$countA++ while $matchstr =~ m/A/gi;
	if ($countA >= $cutoff * $readnum) {$conc = "A"; $count = $countA;}
	
	my $countG = 0;
	$countG++ while $matchstr =~ m/G/gi;
	if ($countG >= $cutoff * $readnum) {$conc = "G"; $count = $countG;}
	
	my $countC = 0;
	$countC++ while $matchstr =~ m/C/gi;
	if ($countC >= $cutoff * $readnum) {$conc = "C"; $count = $countC;}
	
	my $countT = 0;
	$countT++ while $matchstr =~ m/T/gi;
	if ($countT >= $cutoff * $readnum) {$conc = "T"; $count = $countT;}
	
	if ($chrom =~ m/(\d+)/) { $chrom = $1; }
	
	if ($chrom =~ m/mito/) {@cmm[$pos] = "$ref\t$conc\t$readnum\t$count\t$v1"; next;}
	$cm{$chrom}[$pos] = "$ref\t$conc\t$readnum\t$count\t$v1";
	
}
close(MFILEIN);
print "Preparing Reports\n";
my @refzero; #for gap definition.
my @mutzero;
my @results; #for final parsing and reporting.

print SNPFILE "\n$ARGV[0] vs $ARGV[1] SNPS report:\nChromosome\tPosition\tReferenceBase\tWTCalledBase\tWTReadNumber\tWTCalledBaseReadNumber\tWTCalledScore\tMUTCalledBase\tMUTReadNumber\tMUTCalledBaseReadNumber\tMUTCalledScore\n";

my @clength = GetLengths();

foreach (my $chrom = 1; $chrom <= $chroms; $chrom++ ) {
	for (my $x = 1; $x <= $clength[$chrom]; $x++) { #Chromosome length
		if (($cr{$chrom}[$x] eq '') || ($cm{$chrom}[$x] eq '')) { #check for zero reads, store.
			if ($cr{$chrom}[$x] eq '') {push(@refzero, "${chrom}_$x");}
			if ($cm{$chrom}[$x] eq '') {push(@mutzero, "${chrom}_$x");}
			next;
		}
		my @refarr = split(/\t/, $cr{$chrom}[$x]);
		my @mutarr = split(/\t/, $cm{$chrom}[$x]);
		if (($refarr[2] < $minimum) || ($mutarr[2] < $minimum)) {next;}
		my $ref = $refarr[0];
		if ((@refarr[1] ne @mutarr[1]) || (@refarr[1] !~ /$ref/i)) {
			if ((@refarr[1] eq "N") || (@mutarr[1] eq "N")) {next;}
			push(@results, "chromosome$chrom.contig.embl\t$x\t".$cr{$chrom}[$x]."\t".join("\t", @mutarr[1..4]));	
		}
	}
}

for (my $x = 1; $x <= $clength[0]; $x++) { #Mitochondrial length
	if ((@cmr[$x] eq '') || (@cmm[$x] eq '')) {
		if (@cmr[$x] eq '') {push(@refzero, "m_$x");}
		if (@cmm[$x] eq '') {push(@mutzero, "m_$x");}
		next;
	}
	my @refarr = split(/\t/, @cmr[$x]);
	my @mutarr = split(/\t/, @cmm[$x]);
	if (($refarr[2] < $minimum) || ($mutarr[2] < $minimum)) {next;}
	my $ref = $refarr[0];
	if ((@refarr[1] ne @mutarr[1]) || (@refarr[1] !~ /$ref/i)) {
		if ((@refarr[1] eq "N") || (@mutarr[1] eq "N")) {next;}
		push(@results, "mitochondrial.embl\t$x\t".@cmr[$x]."\t".join("\t", @mutarr[1..4]));
	}
}

my $last = scalar @results - 1; #check for no more than 3 SNPs in 10 bases. Dummied.
for (my $x = 0; $x <= $last; $x++) {
#	my @b2 = split(/\t/, @results[$x-2]);
#	my @b1 = split(/\t/, @results[$x-1]);
	my @at = split(/\t/, @results[$x]);
#	my @f1 = split(/\t/, @results[$x+1]);
#	my @f2 = split(/\t/, @results[$x+2]);
#	if ((@at[1] - @b2[1] <=10) && (@at[1] - @b1[1] <= 10)) {next;}
#	if ((@at[1] - @b1[1] <=10) && (@f1[1] - @at[1] <= 10)) {next;}
#	if ((@f1[1] - @at[1] <=10) && (@f2[1] - @at[1] <= 10)) {next;}
	if ((@at[6] <= 12) || (@at[10] <= 12)) {next;} #check quality scores.
	print SNPFILE @results[$x]."\n";
}

close(SNPFILE);
my $zerofile1 = $wdir . $WTname . ".zero";
open(WTZERO,">$zerofile1");

print WTZERO "ZeroCoverage WT\n";

my $prevch = 1;
my $prev = 0;
foreach my $x (@refzero) {
	my @xarr = split("_", $x);
	if ($xarr[1] != $prev + 1) {
		if ($prev == 0) {print WTZERO "$xarr[0]\t$xarr[1]";} else {
			print WTZERO "\t$prev\n$xarr[0]\t$xarr[1]";
		}
	}
	$prev = $xarr[1];
	$prevch = $xarr[0];
}
print WTZERO "\t$prev\n";

close(WTZERO);
my $zerofile2 = $wdir . $MUTname . ".zero";
open(MUTZERO,">$zerofile2");

print MUTZERO "ZeroCoverage MUT\n";

$prevch = 1;
$prev = 0;
foreach my $x (@mutzero) {
	my @xarr = split("_", $x);
	if ($xarr[1] != $prev + 1) {
		if ($prev == 0) {print MUTZERO "$xarr[0]\t$xarr[1]";} else {
			print MUTZERO "\t$prev\n$xarr[0]\t$xarr[1]";
		}
	}
	$prev = $xarr[1];
	$prevch = $xarr[0];
}
print MUTZERO "\t$prev\n";
close(MUTZERO);
}

##Clip zero flanking seqeunce
sub flankclip {
my $coordinates = shift;
my $seqfile = shift;
my $buffer = shift;
my $step = shift;
my $chroms = shift;

my $flankfilen = $wdir . $MUTname . ".flanks";
open(FLFILE,">$flankfilen");

my $seqin = Bio::SeqIO->new(-file => "$seqfile", -format => "fasta" );
my @seqs;
for (my $x = 0; $x <= $chroms; $x++) {
	$seqs[$x] = $seqin->next_seq;
}

open(FILECORR,$coordinates) || die("Could not open ", $coordinates);
while(<FILECORR>)
{
	chomp;
	my @line = split(/\t/, $_);
	if ($line[0] !~ /^\d+/) {next;}
	my $chrom = $line[0];
	my $start = $line[1];
	my $end = $line[2];
	if ($start >= ($step * 2))
	{
		print FLFILE $seqs[$chrom]->subseq($start - ($step + $buffer), $start - $buffer) , "\t$chrom $start left of gap\n"; 
	}
	if ($end > $seqs[$chrom]->length - ($step * 2)) { next;}
	print FLFILE $seqs[$chrom]->subseq($end + $buffer, $end + ($buffer + $step)) , "\t$chrom $end right of gap\n";
}
close(FILECORR);
close(FLFILE);
}

##Parse Gap spanning sequences
sub searchgapreads {
	
my $seedfile = shift;
my $readfile = shift;

local $|=1;
my @flair = ('|', '/', '-', '\\');
my %seeds;
my %seqs;
my $name;
my $read;
my $linecount = 0;
my $i = 0;

print "\n\nReading seeds\n";

open(FILESEQ,$seedfile) || die("Could not open ", $seedfile);
while(<FILESEQ>)
{
	chomp;
	(my $seed, my $position) = split(/\t/, $_);
	$seeds{$seed} = $position;
}
close(FILESEQ);

open(FILEIN,$readfile) || die("Could not open ", $readfile);
while(<FILEIN>)
{
	chomp;
	my $line = $_;
	$linecount++;
	if (($linecount % 4 == 3) || ($linecount % 4 == 0)) {next;}
	if ($linecount % 4 == 1) {
		$name = "$linecount$line";
		next;
	}
	$read = $line;
	$i++;
	print "\rSearching ", $flair[$i % 4];
	foreach my $matchf (keys %seeds)
	{
		my $matchrev = reversecomp($matchf);
		if ($read =~ m/$matchf/gi)
		{
			push(@{ $seqs{$matchf} }, ">$name\n$read\n");
		}
		if ($read =~ m/$matchrev/gi)
		{
			$read = reversecomp($read);
			push(@{ $seqs{$matchf} }, ">$name\n$read\n");
		}
	}
}
close(FILEIN);

print "\nPreparing alignments...\n";
my $dir = $wdir . "align";
system("mkdir ${wdir}align");
foreach my $list (keys %seeds)
{
	my $filename = $wdir. "align/"."$list".".txt";
	open(OUTFILE, ">$filename");
	foreach (@{ $seqs{$list} })
	{
		print OUTFILE $_;
	}
	close(OUTFILE);
}

foreach my $x (keys %seeds)
{
	my $filename = "$x".".txt";
	my $outname = "$x".".aln";
	system("muscle", "-in", "${wdir}align/$filename", "-clwout", "${wdir}align/$outname");
}

system("rm ${wdir}align/*.txt");

opendir(DIR, $dir) || die("Can not open $dir");
my @dirs = readdir(DIR);
closedir(DIR);

open(OUTREPORT, ">$dir/report.txt");

foreach my $file (@dirs) {
	next if ($file !~ /aln/);
	my $seedseq = substr($file, 0, -4);
	open(ALIGN, "$dir/$file");
	print OUTREPORT "$seeds{$seedseq}\n";
	while(<ALIGN>) {
		print OUTREPORT $_;
	}
	print OUTFILE "\n----------------\n";
	close(ALIGN);
}
close(OUTREPORT);

system("rm ${wdir}align/*.aln");

print "\nDONE!\n\n";
}

##Stats from pileups
sub AlignmentStat {

my $pileupfile = shift;
my $reporting = shift;
	
my $sum = 0;
my $coord = 0;
my $minscore = 100;
my $maxscore = 0;
my $sumscore = 0;
my $mind = 50;
my $maxd = 0;
my @vals;
my @valsc;

if ($reporting == 1) {print "Reading File...\n";}
open(SAMFILE,$pileupfile) || die("Could not open ", $pileupfile);
while(<SAMFILE>)
{
	chomp;
	my @line = split(/\t/,$_);
	next if (( $line[0] =~ m/mito/ ) || ($line[7] > 800));
	$coord++;
	$sum = $sum + @line[7];
	push(@vals, $line[7]);
	$sumscore = $sumscore + @line[4];
	push(@valsc, $line[4]);
	$mind = @line[7] if (@line[7] < $mind);
	$maxd = @line[7] if (@line[7] > $maxd);
	$minscore = @line[4] if (@line[4] < $minscore);
	$maxscore = @line[4] if (@line[4] > $maxscore);
}
close(SAMFILE);

my $sqs;
my $sqsc;
my $median;
my $avg = $sum/$coord;
my $avgsc = $sumscore/$coord;
if ($reporting == 1) {print "Calculating STDEV...\n";}
foreach my $x (@vals) {
	$sqs += ($x - $avg) ** 2;
}
foreach my $x (@valsc) {
	$sqsc += ($x - $avgsc) ** 2;
}
my $std = ($sqs / $coord) ** 0.5;
my $stdsc = ($sqsc / $coord) ** 0.5;

#print "Calculating Median...\n";
#my $count = @vals + 1; 
#my @vals = sort { $a <=> $b } @vals; 
#if ($count % 2) { 
# $median = $vals[int($count/2)]; 
#} else { 
# $median = ($vals[$count/2] + $vals[$count/2 - 1]) / 2; 
#}

if ($reporting == 1) {print "\nMin Depth: $mind\nMax Depth: $maxd\nMapped Bases: $sum\nMapped Coordinates: $coord\nAverage Mapped Depth: $avg\nStandard Deviation of Depth: $std\nMin Score: $minscore\nMax Score: $maxscore\nAverage Score: $avgsc\nStandard Deviaiton of Score: $stdsc\n";} #Median Depth: $median\n
if ($reporting == 0) {return $avg;}
}


## Annotation Dependent copy number evaluation
sub AnnotCopy {
my $AVG1 = shift;
my $AVG2 = shift;
my $file1 = shift;
my $file2 = shift;

my $anofilen = $wdir . $MUTname . "_AnCopy.tab";

open(ANOFILE,">$anofilen");

print ANOFILE "File 1 is : $file1 : AVG Depth defined as $AVG1\n";
print ANOFILE "File 2 is : $file2 : AVG Depth defined as $AVG2\n";
#my $lchrom = $ARGV[2];
#print "Chomosome Selected: ", $lchrom, "\n";
#my $reffile = $ARGV[3];
#print "Reference Table: ", $reffile, "\n";
print ANOFILE "\nchrom\tfeat\tstart\tend\tdepth1\tdepth2\tratio\tdesc\n\n";

for (my $loop = 1; $loop <= $chroms; $loop++)
{
	my $lchrom = "chromosome". $loop. ".contig.embl";
	print "Chomosome Selected: ", $lchrom, "\n";
	my $reffile = $wdir . "c". $loop. "table.txt";
	#print "Reference Table: ", $reffile, "\n";

	my @consensusFILE1;
	my @consensusFILE2;

	open(FILE1_PILEUP,$file1) || die("Could not open ".$file1);
	print "\nReading ", $file1, "\n";
	while(<FILE1_PILEUP>)
	 {
		my $line = $_;
		my @parsedItems = split(/\t/, $line);
		my $chrom = shift @parsedItems;
    	if ($lchrom eq $chrom)
    	{
    		my $coord = shift @parsedItems;
    		my $ident = shift @parsedItems;
    		my $called = shift @parsedItems;
    		my $score = shift @parsedItems;
    		my $r0 = shift @parsedItems;
    		my $r1 = shift @parsedItems;
			$consensusFILE1[$coord] = shift @parsedItems;
			#print "$lchrom\t$coord\t$consensusFILE1[$coord]\n";
    	}
	 }
	close(FILE1_PILEUP);

	open(FILE2_PILEUP,$file2) || die("Could not open ".$file2);
	print "Reading ", $file2, "\n";
	while(<FILE2_PILEUP>)
	 {
		my $line = $_;
		my @parsedItems = split(/\t/, $line);
		my $chrom = shift @parsedItems;
	    if ($lchrom eq $chrom)
	    {
	    	my $coord = shift @parsedItems;
	    	my $ident = shift @parsedItems;
	    	my $called = shift @parsedItems;
	    	my $score = shift @parsedItems;
	    	my $r0 = shift @parsedItems;
	    	my $r1 = shift @parsedItems;
			$consensusFILE2[$coord] = shift @parsedItems;
			#print "$lchrom\t$coord\t$consensusFILE2[$coord]\n";
	    }
	 }
	close(FILE2_PILEUP);

	open(FILE3_REF,$reffile) || die("Could not open ".$reffile);
	print "Evaluating ", $reffile, "\n\n";
	while(<FILE3_REF>)
	{
		chomp;
		my $line = $_;
		my ($feat, $start, $end, @desc) = split(/\s+/, $line);
			
		if (($start >= $end) || ($feat eq "gap") ) { next; } #|| ($feat eq "misc_feature")

		my $featcount1 = 0;
		my $featcount2 = 0;
		my $window = $end - $start + 1;
		
		for (my $count = $start; $count <= $end; $count++)
		{
			$featcount1 = $featcount1 + $consensusFILE1[$count];
			$featcount2 = $featcount2 + $consensusFILE2[$count];
		}
	
		if ($featcount2 == 0) { next; }
	
		my $ratio = ($featcount1/$featcount2) * ($AVG2/$AVG1);
		my $depth1 = $featcount1/($window*$AVG1);
		my $depth2 = $featcount2/($window*$AVG2);
	
		if (($depth2 < 0.8) || ($depth1 < 0.8) ) { next; }
	
		print ANOFILE "$lchrom\t$feat\t$start\t$end\t$depth1\t$depth2\t$ratio\t", join(" ", @desc), "\n";
	}
	close(FILE3_REF);
}
close(ANOFILE);
}

##Sliding Window Copy Number Anyalysis - Annotation Independent
sub IndCopy {
my $AVG1 = shift;
my $AVG2 = shift;
my $file1 = shift;
my $file2 = shift;
my $windowsize = shift;
my $ratcutoff = shift;

my $indfilen = $wdir . $MUTname . "_IndCopy.tab";
open(INDFILE, ">$indfilen");

print INDFILE "File 1 is : $file1 : AVG Depth defined as $AVG1\n";
print INDFILE "File 2 is : $file2 : AVG Depth defined as $AVG2\n";
#my $pos1 = $ARGV[5];
#print "Startpoint: ", $pos1, "\n";
#my $pos2 = $ARGV[6];
#print "Endpoint: ", $pos2, "\n";
#my $lchrom = $ARGV[4];
#print "Chomosome Selected: ", $lchrom, "\n";
print INDFILE "Averaging window size: ", $windowsize, " (", floor($windowsize/2), " ", ceil($windowsize/2), ")\n";
print INDFILE "Ratio Cutoff: ", $ratcutoff, " (", (1/$ratcutoff), ")\n";

for( my $loop = 1; $loop <= $chroms; $loop++)
{
	my $lchrom = "chromosome". $loop. ".contig.embl";
	print INDFILE "Chromosome: ", $lchrom, "\n";
	print "Chromosome: ", $lchrom, "\n";
	my @positions = ();
	my @consensusFILE1 = ();
	my @consensusFILE2 = ();

open(FILE1_PILEUP,$file1) || die("Could not open ".$file1);
print "Reading ", $file1, "\n";
while(<FILE1_PILEUP>)
 {
	my $line = $_;
	my @parsedItems = split(/\t/, $line);
	my $chrom = shift @parsedItems;
    if ($lchrom eq $chrom)
    {
    	my $coord = shift @parsedItems;
    	push(@positions, $coord);
    	my $ident = shift @parsedItems;
    	my $called = shift @parsedItems;
    	my $score = shift @parsedItems;
    	my $r0 = shift @parsedItems;
    	my $r1 = shift @parsedItems;
		@consensusFILE1[$coord] = shift @parsedItems;
				
    }
 }
close(FILE1_PILEUP);

open(FILE2_PILEUP,$file2) || die("Could not open ".$file2);
print "Reading ", $file2, "\n";
while(<FILE2_PILEUP>)
 {
	my $line = $_;
	my @parsedItems = split(/\t/, $line);
	my $chrom = shift @parsedItems;
    if ($lchrom eq $chrom)
    {
    	my $coord = shift @parsedItems;
    	my $ident = shift @parsedItems;
    	my $called = shift @parsedItems;
    	my $score = shift @parsedItems;
    	my $r0 = shift @parsedItems;
    	my $r1 = shift @parsedItems;
		@consensusFILE2[$coord] = shift @parsedItems;
		
    }
 }
close(FILE2_PILEUP);

my $basecount = 0;
#my $localbasecount = $pos2 - $pos1 + 1;
my $totalcount1 = 0;
my $totalcount2 = 0;
#my $subcount1 = 0;
#my $subcount2 = 0;
my $max = 1;
my $maxc;
my $min = 1;
my $minc;
my $flaggedbases = 0;
my $consec;
my $conslen;
my $ratio;
my $s1;
my $s2;
my $w1;
my $w2;

print "Parsing Data\n";
print INDFILE "\nPosition\tLocalRatio\tWindowRatio\tLength\tAvg Depth 1\tAvg Depth 2\tWAvg Depth 1\tWAvg Depth 2\tChromosome\n";
	foreach (@positions)
	 {
	 	my $coord = $_;
	 	if ($coord < floor($windowsize/2))
	 	{
	 		$basecount++;
	 		$totalcount1 += @consensusFILE1[$coord];
	 		$totalcount2 += @consensusFILE2[$coord];
	 		next;
	 	}
	 	my $readcount1 = 0;
	 	my $readcount2 = 0;
	 	for (my $count = $coord - floor($windowsize/2); $count <= $coord + ceil($windowsize/2); $count++)
	 	{
	 		$readcount1 += @consensusFILE1[$count];
	 		$readcount2 += @consensusFILE2[$count];
	 	}
	 	
	 	if($readcount2 == 0) {next;}
	 	
	 	$ratio = (($readcount1/$readcount2) * ($AVG2/$AVG1));
	 	if ($ratio > $max) {
	 		$max = $ratio;
	 		$maxc = $coord;
	 	}
	 	if ($ratio < $min) {
	 		$min = $ratio;
	 		$minc = $coord;	 		
	 	}
	 		 	 		 	
	 	$totalcount1 += @consensusFILE1[$coord];
	 	$totalcount2 += @consensusFILE2[$coord];
	 	
	 	#if (($coord >= $pos1) && ($coord <= $pos2)) #subsequence
	 	#{
	 	#	$subcount1 += @consensusFILE1[$coord];
	 	#	$subcount2 += @consensusFILE2[$coord];
	 	#}
	 	
	 	$basecount++;
	 	if (($ratio >= $ratcutoff) || ($ratio <= (1/$ratcutoff)))
	 	{
	 		$flaggedbases++;
	 		if ($consec != $coord - 1)
	 		{
	 			$s1 = 0;
	 			$s2 = 0;
	 			for (my $count = $consec - $conslen; $count <= $consec; $count++)
	 			{
	 				$s1 += @consensusFILE1[$count];
	 				$s2 += @consensusFILE2[$count];
	 			}
	 			
	 			$w1 = 0;
	 			$w2 = 0;
	 			for (my $count = $consec - $conslen - floor($windowsize/2); $count <= $consec + ceil($windowsize/2); $count++)
	 			{
	 				$w1 += @consensusFILE1[$count];
	 				$w2 += @consensusFILE2[$count];
	 			}
	 				 			
	 			if ($s2 == 0)
	 			{
	 				print INDFILE "\n", $coord, "\t";
	 			} else {
	 				my $sratio = ($s1 / $s2) * ($AVG2/$AVG1);
	 				my $wratio = ($w1 / $w2) * ($AVG2/$AVG1);
	 				print INDFILE $sratio, "\t", $wratio, "\t", $conslen, "\t", $s1/($conslen*$AVG1), "\t", $s2/($conslen*$AVG2), "\t", $w1/(($conslen+$windowsize)*$AVG1), "\t", $w2/(($conslen+$windowsize)*$AVG2), "\t", $lchrom, "\t\n", $coord, "\t";
	 			}
	 			$consec = $coord;
	 			$conslen = 1;
	 		} else {
	 			$conslen++;
	 			$consec = $coord;
	 		}
	 	}
	 }	 
 
 
 $s1 = 0;
 $s2 = 0;
 for (my $count = $consec - $conslen; $count <= $consec; $count++)
 {
	 $s1 += @consensusFILE1[$count];
	 $s2 += @consensusFILE2[$count];
 }
$w1 = 0;
$w2 = 0;
for (my $count = $consec - $conslen - floor($windowsize/2); $count <= $consec + ceil($windowsize/2); $count++)
{
$w1 += @consensusFILE1[$count];
$w2 += @consensusFILE2[$count];
}
if ($s2 == 0)
{
 	print INDFILE "\n";
 } else {
 	my $sratio = ($s1 / $s2) * ($AVG2/$AVG1);
 	my $wratio = ($w1 / $w2) * ($AVG2/$AVG1);
 	print INDFILE $sratio, "\t", $wratio, "\t", $conslen, "\t", ($s1/($conslen*$AVG1)), "\t", ($s2/($conslen*$AVG2)), "\t", ($w1/(($conslen+$windowsize)*$AVG1)), "\t", ($w2/(($conslen+$windowsize)*$AVG2)), "\t", $lchrom, "\n";
 }
 
 print "\nComplete!  ", $basecount, " bases read from ", $lchrom, "\n";
 print "Average depth of File 1: ", ($totalcount1/$basecount), "\n";
 print "Average depth of File 2: ", ($totalcount2/$basecount), "\n";
 #print "Region of ", $localbasecount, " bases tested...\n";
 #print "Average depth of Defined Region in File 1: ", ($subcount1/$localbasecount), "\n";
 #print "Average depth of Defined Region in File 2: ", ($subcount2/$localbasecount), "\n";
 #print "Corrected ratio for selected region : ", ($subcount1/$subcount2) * 1.2, "\n";
 print "Maximum ratio of ", $max, " encountered at position ", $maxc, "\n";
 print "Minimum ratio of ", $min, " encountered at position ", $minc, "\n";
 print "Total of ", $flaggedbases, " positions found.\n";
}
close(INDFILE);
}

##Annotation of called SNPS
sub Annotate {
my $SNPfile = shift;
my $merLen   = 13;
my $upstream = 400;

my @contigList = ();
for (my $x = 1; $x <= $chroms; $x++) {
	my $contig = "chromosome".$x.".contig";
	push(@contigList, $contig);
}
push(@contigList, "mitochondrial");

my $manualAnnotFile = "RM-manual-annotations.txt";

my $SNPrfile = $wdir . $MUTname . "_Annotated.SNPS";
open(ANOTREP,">$SNPrfile");

my %snpResult;
my %snp2Genes;
my %snpAnnotation;


sub diffline {
    my ($str1,$str2) = @_;

    my $i = 0;
    my $max = length($str1);
    $max = length($str2) if $max > length($str2);
    for ($i = 0; $i < $max; $i++) {
	 return $i if (substr($str1,$i,1) ne substr($str2,$i,1));
    }
    return -1;
}

sub binarySearch {
    my ( $query, $arList ) = @_;

    my $low  = 0;
    my $high = @{$arList} - 1;
    my $mid;

    while ( $low <= $high ) {
        $mid = int( ( $low + $high ) / 2 );
        if ( $query < $$arList[$mid] ) {
            $high = $mid - 1;
        }
        else {
            $low = $mid + 1;
        }
    }
    $mid++ if ( $query > $$arList[$mid] );

    return $mid;
}

my %manualAnnots;
if (open ANNOTFILE,"<$manualAnnotFile") {
	while (<ANNOTFILE>) {
		chomp;
		my ($gene,$manualAnnot) = split /\t/,$_;
		$manualAnnots{$gene} = $manualAnnot;
	}
	close ANNOTFILE;
}

open(MFILEIN,$SNPfile);
while (<MFILEIN>) {
    chomp;
    my ( $contig, $pos, $refBase, $wtBase, $wtReads, $wtCount, $wtScore, $mutBase, $mutReads, $mutCount,
        @rest ) = split /\t/, $_;
    if ($contig =~ m/mito/) { $contig = "mitochondrial.embl";}
    next unless ($contig =~ m/embl/);
    @{ $snpResult{$contig}{$pos} } =
      ( $refBase, $wtBase, $wtCount, $mutBase, $mutCount, $wtReads, $mutReads, @rest );
}
close(MFILEIN);

print ANOTREP
"Chromosome\tPosition\tRefBase\tWTbase\tWTReads\tWTcount\tMUTbase\tMUTReads\tMUTcount\tNonsynomity\tAnnotations\tGene1\tGene2\tGene3\n";

foreach my $chrom (@contigList) {
	my $ifile = $wdir . $chrom . ".embl";
	my $chromz = $chrom . ".embl";
    my $seqio_object       = Bio::SeqIO->new( -file => $ifile );
    my $seq_object         = $seqio_object->next_seq;
    my @topfeatures        = $seq_object->top_SeqFeatures();
    my $lengthOfChromosome = $seq_object->length;
    my $orig_seq           = $seq_object->seq();
    my %taggedSNPs;

    my @snpPositions = keys %{ $snpResult{$chromz} };

    push @snpPositions, -1;
    push @snpPositions, 999999999;
    @snpPositions = sort { $a <=> $b } @snpPositions;

    foreach my $feat ( $seq_object->top_SeqFeatures ) {

        if ( $feat->primary_tag eq 'CDS' ) {
            my @genes = ();
	    my $snpHasGenes = 0;

            if ( $feat->has_tag('gene') ) {
                for my $val ( $feat->get_tag_values('gene') ) {
                    push @genes, $val;
                }
            }

            if ( $feat->location->isa('Bio::Location::Split') ) {

                #      print "Split: " . $feat->primary_tag;
                foreach my $location ( $feat->location->sub_Location ) {

                    #print $location->start , ".." , $location->end, "\n";
                    my $startIndex =
                      binarySearch( $location->start, \@snpPositions );
                    my $endIndex =
                      binarySearch( $location->end, \@snpPositions );
                    if ( $endIndex > $startIndex )
                    {    # there's an interventing SNP
                        my $snpPos = $snpPositions[$startIndex];
                        $snp2Genes{$chromz}{$snpPos} = \@genes;
	                $snpHasGenes = $snpPos;
                    }
                }
            }

            if ( $feat->location->isa('Bio::Location::Simple') ) {

#	      print "Simple location: " . $feat->location->start . ".." . $feat->location->end . "\n";
                my $dbg;
            }

            my $startIndex =
              binarySearch( $feat->location->start, \@snpPositions );
            my $endIndex = binarySearch( $feat->location->end, \@snpPositions );
            my $dbg;
            if ( $endIndex > $startIndex ) {    # there's an interventing SNP
                my $snpPos = $snpPositions[$startIndex];
                $snp2Genes{$chromz}{$snpPos} = \@genes;
                $snpHasGenes = $snpPos;

            }

	    if ($snpHasGenes) {
		my $pos = $snpHasGenes;

                my ( $refBase, $wtBase, $wtCount, $mutBase, $mutCount, $wtReads, $mutReads, @rest ) = @{ $snpResult{$chromz}{$pos} };
		if ($refBase eq $wtBase) {
                    my $cds_obj = $feat->spliced_seq;
    	            my $cds1 = $cds_obj->seq . ""; # paranoia: append null string to force string evaluation

		    # Create a temporary DNA sequence with our single base mutated.  This
		    # will be used to determine the protein product.  Especially for
		    # spliced regions, this approach is much simpler (although slower
		    # computationally) than developing the necessary logic for searching
		    # for non-synonymous mutations.
		    my $temp_seq = substr($orig_seq,0,$pos-1) . $mutBase . substr($orig_seq,$pos);
		    $seq_object->seq($temp_seq);
                    my $cds_obj2 = $feat->spliced_seq;
    	            my $cds2 = $cds_obj2->seq . ""; # paranoia: append null string to force string evaluation
		    $seq_object->seq($orig_seq); # restore the original sequence
		    if ($cds1 ne $cds2) {
			# calculate the translated protein sequences for the original
			# and mutated DNA sequences
			my $seq1 = new Bio::PrimarySeq(-seq => $cds1);
			my $tr1 = $seq1->translate()->seq();
			my $seq2 = new Bio::PrimarySeq(-seq => $cds2);
			my $tr2 = $seq2->translate()->seq();
			if ($tr1 ne $tr2) {
				my $dnachar = diffline($cds1,$cds2);
				my $annot = substr($cds1,$dnachar,1) . ($dnachar+1) . substr($cds2,$dnachar,1);
				if (length($tr1) ne length($tr2)) {
				    $annot .= ";TRUNC(" . length($tr1) . "," . length($tr2) . ")";
				} else {
				    my $aachar = diffline($tr1,$tr2);
				    $annot .= ";";
				    $annot .= substr($tr1,$aachar,1) . ($aachar+1) . substr($tr2,$aachar,1);
				}
				$snpAnnotation{$chromz}{$pos} = $annot;
			} else {
				$snpAnnotation{$chromz}{$pos} = "synonymous";
			}
		    } else {
			$snpAnnotation{$chromz}{$pos} = "spliced sequence unchanged";
		    }
		} else {
		    $snpAnnotation{$chromz}{$pos} = "ref != WT;";
		    		my $cds_obj = $feat->spliced_seq;
    	            my $cds1 = $cds_obj->seq . ""; # paranoia: append null string to force string evaluation

		    # Create a temporary DNA sequence with our single base mutated.  This
		    # will be used to determine the protein product.  Especially for
		    # spliced regions, this approach is much simpler (although slower
		    # computationally) than developing the necessary logic for searching
		    # for non-synonymous mutations.
		    my $temp_seq = substr($orig_seq,0,$pos-1) . $mutBase . substr($orig_seq,$pos);
		    $seq_object->seq($temp_seq);
                    my $cds_obj2 = $feat->spliced_seq;
    	            my $cds2 = $cds_obj2->seq . ""; # paranoia: append null string to force string evaluation
		    $seq_object->seq($orig_seq); # restore the original sequence
		    if ($cds1 ne $cds2) {
			# calculate the translated protein sequences for the original
			# and mutated DNA sequences
			my $seq1 = new Bio::PrimarySeq(-seq => $cds1);
			my $tr1 = $seq1->translate()->seq();
			my $seq2 = new Bio::PrimarySeq(-seq => $cds2);
			my $tr2 = $seq2->translate()->seq();
			if ($tr1 ne $tr2) {
				my $dnachar = diffline($cds1,$cds2);
				my $annot = substr($cds1,$dnachar,1) . ($dnachar+1) . substr($cds2,$dnachar,1);
				if (length($tr1) ne length($tr2)) {
				    $annot .= ";TRUNC(" . length($tr1) . "," . length($tr2) . ")";
				} else {
				    my $aachar = diffline($tr1,$tr2);
				    $annot .= ";";
				    $annot .= substr($tr1,$aachar,1) . ($aachar+1) . substr($tr2,$aachar,1);
				}
				$snpAnnotation{$chromz}{$pos} .= $annot;
			} else {
				$snpAnnotation{$chromz}{$pos} .= "synonymous";
			}
		    } else {
			$snpAnnotation{$chromz}{$pos} .= "spliced sequence unchanged";
		    }
		}
	     }

#      print "$chromz CDS sequence begins ",substr($cds_obj->seq,0,30)," orfs: ";
#            my @orfs = find_orfs( $cds_obj->seq, 0, 1, 90 );
#            foreach my $orf (@orfs) {
#                print join(",",@{$orf}) . ";";
#            }
#      print "\n";
        }

    }

    foreach my $pos ( sort { $a <=> $b } keys %{ $snpResult{$chromz} } ) {
        my ( $refBase, $wtBase, $wtCount, $mutBase, $mutCount, $wtReads, $mutReads, @rest ) =
          @{ $snpResult{$chromz}{$pos} };
	my @genes;
	my $genes;

        if ( $snp2Genes{$chromz}{$pos} ) {
            @genes = sort @{ $snp2Genes{$chromz}{$pos} };
            $genes = join( ",", @genes );
	}

	{
	    my $manualAnnot = "";
            $_ = $chromz;
            s/^chromosome//;
            s/\.contig.*//;

            foreach my $gene (@genes) {
	        $manualAnnot = $manualAnnots{$gene} if ($manualAnnots{$gene});
	    }

            my $note = "SNP;RefBase:$refBase;WT:$wtBase,$wtReads,$wtCount;MUT:$mutBase,$mutReads,$mutCount";
	    $note .= ";genes:$genes" if $genes;

            print ANOTREP "$_\t$pos\t$refBase\t$wtBase\t$wtReads\t$wtCount\t$mutBase\t$mutReads\t$mutCount\t";

	    if ($snpAnnotation{$chromz}{$pos}) {
	        print ANOTREP $snpAnnotation{$chromz}{$pos};
		$note .= ";NONSYNON: " . $snpAnnotation{$chromz}{$pos};
	    }

            print ANOTREP "\t$manualAnnot";
	    
            foreach my $gene (@genes) {
                print ANOTREP
"\t=Hyperlink(\"http://www.uniprot.org/uniprot/?query=$gene&sort=score\",\"$gene\")";
            }
            print ANOTREP "\n";

            my $feat = new Bio::SeqFeature::Generic(
                -start   => $pos,
                -end     => $pos,
                -strand  => 1,
                -primary => 'polymorphism',
                -tag     => {
                    evidence => 'experimental',
                    note => $note
                }
            );
            $seq_object->add_SeqFeature($feat);
        }

    }

    my $seqio_out_obj =
      Bio::SeqIO->new( -file => ">$wdir$chrom.out.embl", -format => 'embl' );
    $seqio_out_obj->write_seq($seq_object);
}
close(ANOTREP);
}

#FASTA and FEAT table Construction
sub refbuild {
my $fastafil = $BTpath."genomes/".$refGen.".fasta";
print "Fasta to $fastafil\n";
open(FASTAFILE,">$fastafil");
print "Preparing FASTA and feature tables from:\n";
my @contigList = ();
push(@contigList, "mitochondrial.embl");
for (my $x = 1; $x <= $chroms; $x++) {
	my $contig = "chromosome".$x.".contig.embl";
	push(@contigList, $contig);
}
my @lengths;
my $i = 0;
foreach my $chrom (@contigList) {
    $i++;
    my $cfile = "$wdir$chrom";
    print "$cfile\n";
    my $seqio_object       = Bio::SeqIO->new( -file => $cfile );
    my $seq_object         = $seqio_object->next_seq;
    my $lengthOfChromosome = $seq_object->length;
    my $orig_seq           = $seq_object->seq();
    print FASTAFILE ">$chrom\n";
    my $iter = int($lengthOfChromosome / 1000);
    for (my $z = 1; $z <= $iter; $z++) {
    	my $subse = $seq_object->subseq((($z-1)*1000 + 1), $z*1000);
    	print FASTAFILE "$subse\n";
    }
    my $subse = $seq_object->subseq(($iter*1000 + 1), $lengthOfChromosome);
    print FASTAFILE "$subse\n";
    if ($i <= $chroms) {open(TABLEFILE,">${wdir}c${i}table.txt");}
    else {open(TABLEFILE,">${wdir}cmtable.txt");}
    push(@lengths, $lengthOfChromosome);
	for my $feat_object ($seq_object->get_SeqFeatures) {
		my ($primary, $start, $end, $comp, @gene);
		$primary = $feat_object->primary_tag;
		$start = $feat_object->location->start;
		$end = $feat_object->location->end;
		$comp = $feat_object->location->strand;
		if ($comp == 1) {$comp = "";}
		if ($comp == -1) {$comp = "c";}
		for my $tag ($feat_object->get_all_tags) {
			push (@gene, $feat_object->get_tag_values("gene")) if ($feat_object->has_tag("gene"));
			push (@gene, $feat_object->get_tag_values("product")) if ($feat_object->has_tag("product"));
			push (@gene, $feat_object->get_tag_values("note")) if ($feat_object->has_tag("note"));
		}
		print TABLEFILE "$primary\t$start\t$end\t$comp @gene\n";
	}
	close(TABLEFILE);
}
close(FASTAFILE);
print "Done!\n";
return @lengths;
}

#get and return chromosome lengths
sub GetLengths {
	my @contigList = ();
	push(@contigList, "mitochondrial.embl");
	for (my $x = 1; $x <= $chroms; $x++) {
		my $contig = "chromosome".$x.".contig.embl";
		push(@contigList, $contig);
	}
	my @lengths;
	foreach my $chrom (@contigList) {
       	my $cfile = "$wdir$chrom";
    	my $seqio_object       = Bio::SeqIO->new( -file => $cfile );
    	my $seq_object         = $seqio_object->next_seq;
    	my $lengthOfChromosome = $seq_object->length;
    	push(@lengths, $lengthOfChromosome);
	}
    return @lengths;
}

#Performs alignments and converstions to sorted BAM and Pileups
sub RAlign {
	my $readf = shift;
	my $target = shift;
	my $named = shift;
	print "\n\nRunning alignment of $named\n";
	system("${BTpath}bowtie -S $refGen ${BTpath}reads/$readf ${wdir}${target}.sam");
	print "\n\nRunning BAM conversion of $named\n";
	system("${SAMpath}samtools view -bS -o ${wdir}${target}.bam ${wdir}${target}.sam");
	system("rm ${wdir}${target}.sam");
	print "\n\nRunning Sort of $named\n";
	system("${SAMpath}samtools sort ${wdir}${target}.bam ${wdir}${target}.sorted");
	system("rm ${wdir}${target}.bam");
	print "\n\nRunning indexing of $named\n";
	system("${SAMpath}samtools index ${wdir}${target}.sorted.bam");
	print "\n\nRunning pileup of $named\n";
	system("${SAMpath}samtools pileup -f ${BTpath}genomes/${refGen}.fasta -c ${wdir}${target}.sorted.bam > ${wdir}${target}.pileup");
}

#Other
=head1 NAME

pombetools.pl

=head1 DESCRIPTION

The pombe toolset has been developed to facilitate the evaluation of whole genome sequencing data from yeasts.  Currently, the toolset is limited to pombe strains, however the toolset is easily scalable to other yeasts or other sequenced genomes.  The included tools assist in the identification of SNP mutations, short (and long) insertion/deletions, and changes in gene/region copy number between a parent strain and mutant or revertant strain.

Currently, the toolset is run from the command line in a unix or similar terminal and requires a few additional programs to run noted below.

=head2 OPTIONS

In addition to the options obtained from config.txt, the following command line options are used:

=over 4

=item –build

Reads genome embl files from the working directory to construct fasta and indexes for alignment as well as feature index tables for targeted evaluation of changes in copy number.  Should be run first unless the same genome has been built previously without modifications.

=item -all

Executes all subroutines described below.

=item -align

Performs alignment of WT and MUT reads to a built genome.  Generates “sorted.bam” and index files suitable for viewing alignments in a program like IGV (http://www.broadinstitute.org/igv/).  Generates plaintext pileup files suitable for subsequent parsing or by-eye evaluation of alignments.

=item -stats

Provides some basic statistics related to an alignment such as average mapped read depth.

=item -SNP

Calls single nucleotide polymorphisms (SNPs) from WT and MUT alignments.  SNPs are called relative to the provided reference genome as well as between WT and MUT.

=item -flanks

Parses MUT.zero file and clips sequence flanks to attempt to resolve insertion/deletion mutations.

=item -gaps

Utilizes a clipped flank file to search reads for gap spanning sequence.  Combines reads and aligns them with the MUSCLE alignment tool to generate report.txt in the align subdirectory.

=item -AnCopy

Utilizes mapped read depth of WT and MUT across defined sequence elements such as protein coding genes to identify changes in copy number.

=item -InCopy

Scans across WT and MUT genomes to identify regions in a context-insensitive manner that demonstrate a change in copy number.  Flags bases using a scanning averaging window across each chromosome that exceed a cutoff fold-change in mapped read depth.

=back

=head1 TO DO

Portability to other genomes.  Inteface cleanup.  Overhaul of gap resolution technique.

=head1 AUTHORS

=over 4

=item
Jonathan Epstein, E<lt>Jonathan_Epstein@nih.govE<gt>

=item
James Iben, F<lt>ibenjame@mail.nih.govE<gt>

=back

=head1 COPYRIGHT AND LICENSE

                          PUBLIC DOMAIN NOTICE

        National Institute of Child Health and Human Development

 This software/database is a "United States Government Work" under the
 terms of the United States Copyright Act.  It was written as part of
 the author's official duties as a United States Government employee and
 thus cannot be copyrighted.  This software/database is freely available
 to the public for use. The National Institutes of Health and the U.S.
 Government have not placed any restriction on its use or reproduction.

 Although all reasonable efforts have been taken to ensure the accuracy
 and reliability of the software and data, the NIH and the U.S.
 Government do not and cannot warrant the performance or results that
 may be obtained by using this software or data. The NIH and the U.S.
 Government disclaim all warranties, express or implied, including
 warranties of performance, merchantability or fitness for any particular
 purpose.
 
Please cite the author in any work or product based on this material.
 
 
This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.5 or,
at your option, any later version of Perl 5 you may have available.

=cut